#!/usr/bin/env python

'''Cobalt Queue Status'''
__revision__ = '$Revision: 1.11 $'

from sss.ssslib import comm_lib, LookupError as SSSLookupError
from getopt import getopt, GetoptError
from math import floor
from sys import argv
from elementtree.ElementTree import XML
from time import ctime, mktime, localtime, strftime, time
from datetime import datetime
from Cobalt.Util import print_tabular

def convert_minutes(thetime):
    '''Converts input in minutes to hh:mm:00 format
    thetime is the minute string
    returns string of thetime in hh:mm:00 format
    '''
    thetime = str(thetime)
    t = int(thetime.split('.')[0])
    h = int(floor(t/60))
    t -= (h * 60)
    return "%02d:%02d:00" % (h, t)

if __name__ == '__main__':
    try:
        (opts, args) = getopt(argv[1:], 'df', ['version'])
    except GetoptError, msg:
        print "Usage: cqstat [--version] [-d] [-f jobid]"
        print msg
        raise SystemExit, 1

    if '--version' in argv:
        print "cqstat %s" % __revision__
        raise SystemExit, 0

    jobid = None

    if len(args) > 0:
        jobid = args[0]

    if jobid:
        jobid_tosend = jobid
    else:
        jobid_tosend = "*"

    comm = comm_lib(debug=('-d' in argv))

    try:
        handle = comm.ClientInit("queue-manager")
    except SSSLookupError:
        print "failed to connect to queue manager"
        raise SystemExit, 1
    if '-f' in argv:
        msg = "<get-job><job user='*' walltime='*' nodes='*' state='*' jobid='%s' location='*' mode='*' procs='*' queue='*' starttime='*'/></get-job>"
    else:
        msg = "<get-job><job user='*' walltime='*' nodes='*' state='*' jobid='%s' location='*'/></get-job>" 
    comm.SendMessage(handle, msg % jobid_tosend)
    xdata = comm.RecvMessage(handle)
    comm.ClientClose(handle)
    resp = XML(xdata)

    if '-f' in argv:
        header = [['JobID', 'User', 'WallTime', 'Nodes', 'State', 'Location', 'Mode', 'Procs', 'Queue', 'StartTime']]
    else:
        header = [('JobID', 'User', 'WallTime', 'Nodes', 'State', 'Location')]

    # get list of attributes per job
    output = [[job.get(x) for x in [y.lower() for y in header[0]]] for job in resp.findall('.//job')]

    if '-f' in argv:
        header[0].append('RunTime')

    if output:
        maxjoblen = max([len(item[0]) for item in output])
        jobidfmt = "%%%ss" % maxjoblen
    # next we cook walltime
    for i in xrange(len(output)):
        if '-f' in argv:
            # RunTime and StartTime
            if output[i][9] == '-1' or output[i][9] == 'BUG':
                output[i].append('N/A')  # RunTime
                output[i][9] = 'N/A'     # StartTime
            else:
                runtime = datetime.fromtimestamp(time()) - datetime.fromtimestamp(float(output[i][9]))
                minutes, seconds = divmod(runtime.seconds, 60)
                hours, minutes = divmod(minutes, 60)
                output[i].append( "%02d:%02d:%02d" % (hours, minutes, seconds) )

                output[i][9] = strftime("%m/%d/%y %T", localtime(float(output[i][9])))

        # walltime
        output[i][2] = convert_minutes(output[i][2])
        # jobid
        output[i][0] = jobidfmt % output[i][0]
    output.sort()
    print_tabular([tuple(x) for x in header + output])

